"""This module provide functions that are related to giving iris information from geographical data."""

import pandas as pd
import geopandas as gpd
from iris_insee_utils.get_iris_contours_data import read_or_download_iris_contour_data
import iris_insee_utils


def gps_to_iris_number(long: float, lat: float, iris_year: int = 2018):
    """
    Get the longitude and latitude of gps point(s), and returns the CODE IRIS.

    Parameters
    ----------
    long : float
        Longitude of the GPS point.
    lat : float
        Latitude of the GPS point.
    iris_year : int, optional
        Year of the IRIS data to use. Default is 2018.

    Returns
    -------
    str
        The CODE IRIS.

    Examples
    --------
    >>> gps_to_iris_number(5.36, 43.41, 2018)
    Out[11]:
    '130710101'
    """
    return gps_to_iris(long, lat, iris_year).CODE_IRIS.values[0]
    

def gps_to_iris(
    long: float, lat: float, iris_year: int = 2018, iris_full_info: bool = False
) -> gpd.GeoDataFrame:
    """
    Get the longitude and latitude of gps point(s), and returns the CODE IRIS.
    More information about TYPE_IRIS can be found here: https://www.insee.fr/fr/information/2438155

    Parameters
    ----------
    long : float
        Longitude of the GPS point.
    lat : float
        Latitude of the GPS point.
    iris_year : int, optional
        Year of the IRIS data to use. Default is 2018.
    iris_full_info : bool, optional
        If True, return the full information of the IRIS (name, ). Default is False.

    Returns
    -------
    geopandas.GeoDataFrame
        A GeoDataFrame with one row containing the CODE IRIS.

    Examples
    --------
    >>> gps_to_iris(5.362223663076667, 43.41522600589665, 2018)
    Out[11]:
           CODE_IRIS           NOM_IRIS    TYP_IRIS
        0  130710101  Cd6-Plan de Campagne        A
    """
    df_ign_map = read_or_download_iris_contour_data(iris_year)
    df_ign_map = df_ign_map.to_crs(epsg=4326)
    gdf = gpd.GeoDataFrame(geometry=gpd.points_from_xy([long], [lat]))

    gdf.crs = "EPSG:4326"
    gdf = gdf.to_crs(epsg=4326)
    result_df = gpd.sjoin(gdf.head(1), df_ign_map, predicate="within")
    if iris_full_info:
        return result_df
    return result_df[["CODE_IRIS", "NOM_IRIS", "TYP_IRIS"]]


def df_gps_to_iris(
    df: pd.DataFrame,
    long_col: str,
    lat_col: str,
    iris_year: int = 2018,
    iris_full_info: bool = False,
) -> gpd.GeoDataFrame:
    """
    Get the longitude and latitude from a DataFrame, and returns the CODE IRIS.
    More information about TYPE_IRIS can be found here: https://www.insee.fr/fr/information/2438155

    Parameters
    ----------
    df : pd.DataFrame
        DataFrame containing the GPS points.
    long_col : str
        Name of the column containing the longitude.
    lat_col : str
        Name of the column containing the latitude.
    iris_year : int, optional
        Year of the IRIS data to use. Default is 2018.
    iris_full_info : bool, optional
        If True, return the full information of the IRIS (name, ). Default is False.

    Returns
    -------
    geopandas.GeoDataFrame
        A GeoDataFrame with rows containing the CODE IRIS for each GPS point.

    Examples
    --------
    >>> df = pd.DataFrame({'Lieu':['Mairie de Marseille','Site-Mémorial du Camp des Milles'],'longitude': [5.369905252590892, 5.382786610618382,], 'latitude': [43.296630332564405,43.5034655315141,]})
    >>> df_gps_to_iris(df, 'longitude', 'latitude', 2018)
    Out[11]:
                       geometry  index_right INSEE_COM  ...  CODE_IRIS              NOM_IRIS TYP_IRIS
    0  POINT (5.36222 43.41523)        37408     13071  ...  130710101  Cd6-Plan de Campagne        A
    """
    # Check if the columns are not already in the dataframe
    conflicting_cols = [
        col for col in ["RIS", "NOM_IRIS", "CODE_IRIS"] if col in df.columns
    ]
    assert (
        not conflicting_cols
    ), f"{', '.join(conflicting_cols)} are already in the inputed dataframe columns, please rename them or drop them to avoid conflicts"

    df_ign_map = gpd.read_parquet(
        iris_insee_utils.__path__[0] + f"/../data/transformed/iris_{iris_year}.parquet"
    )
    df_ign_map = df_ign_map.to_crs(epsg=4326)
    df = df.astype(
        {long_col: str, lat_col: str}
    )  # TODO: Check if categorical or string dtype would not be faster or better.
    gdf = gpd.GeoDataFrame(df, geometry=gpd.points_from_xy(df[long_col], df[lat_col]))

    gdf.crs = "EPSG:4326"
    gdf = gdf.to_crs(epsg=4326)
    result_df = gpd.sjoin(gdf, df_ign_map, predicate="within")
    if iris_full_info:
        return result_df
    return result_df.drop(
        columns=["geometry", "index_right", "INSEE_COM", "NOM_COM", "TYP_IRIS"]
    )



import folium
import unittest
from iris_insee_utils.iris_map import plot_folium_map
import pandas as pd
import os


class TestFoliumMap(unittest.TestCase):

    def test_plot_folium_map(self):
        # Call the function to generate the map
        m = plot_folium_map(commune_name="Nice", iris_year=2018)

        # Check if the returned object is a Folium Map
        self.assertIsInstance(m, folium.Map)

        # Generate HTML content
        html_content = m.get_root().render()

        print(html_content[:2000])
        # Check if the HTML content contains expected elements
        self.assertIn('<div class="folium-map"', html_content)
        self.assertIn("Nice", html_content)  # Check if the commune name is in the HTML

    def test_plot_folium_map_with_df_oi(self):
        df_oi = pd.DataFrame(
            {
                "Lieu": ["Mairie de Marseille", "Site-Mémorial du Camp des Milles"],
                "lon": [5.369905252590892, 5.382786610618382],
                "lat": [43.296630332564405, 43.5034655315141],
            }
        )

        m = plot_folium_map(iris_year=2018, commune_name="Marseille", df_oi=df_oi)
        self.assertIsInstance(m, folium.Map)

    def test_plot_folium_map_df_enrich(self):
        df_oi = pd.DataFrame(
            {
                "Lieu": ["Mairie de Marseille", "Site-Mémorial du Camp des Milles"],
                "lon": [5.369905252590892, 5.382786610618382],
                "lat": [43.296630332564405, 43.5034655315141],
            }
        )

        df_enrich = pd.DataFrame(
            {"CODE_IRIS": ["132020301", "130010905"], "Some_IRIS_Data": [100, 200]}
        )

        save_map_path = "test_map.html"

        m = plot_folium_map(
            iris_year=2018,
            commune_name="Marseille",
            df_oi=df_oi,
            df_enrich=df_enrich,
            df_enrich_iriscol_colname="CODE_IRIS",
            df_enrich_select_cols=["Some_IRIS_Data"],
        )

        assert isinstance(m, folium.Map)


def test_plot_folium_map():
    save_map_path = "test_map.html"
    m = plot_folium_map(
        commune_name="Nice", iris_year=2018, save_map_path=save_map_path
    )

    assert isinstance(m, folium.Map)
    assert os.path.exists(save_map_path)

    # Clean up
    os.remove(save_map_path)


if __name__ == "__main__":
    unittest.main()



"""This module tests that GPS coordinates are linking to the correct IRIS."""

import pandas as pd

from iris_insee_utils.gps_coordinates_to_iris import gps_to_iris, df_gps_to_iris, gps_to_iris_number



def test_gps_to_iris():
    """
    Test that a single GPS point outputs the correct IRIS.
    """
    result_df = gps_to_iris(5.36413, 43.23871)
    assert result_df.CODE_IRIS.values[0] == "132080302"


def test_gps_to_iris_number():
    """
    Test that a single GPS point outputs the correct IRIS number.
    """
    result_str = gps_to_iris_number(5.362223663076667, 43.41522600589665, 2018)
    assert result_str == "130710101"

def test_df_gps_to_iris():
    """
    Test that GPS points in a DataFrame output the correct IRIS.
    """
    df = pd.DataFrame(
        {
            "Lieu": ["Mairie de Marseille", "Site-Mémorial du Camp des Milles"],
            "longitude": [5.369905252590892, 5.382786610618382],
            "latitude": [43.296630332564405, 43.5034655315141],
        }
    )
    result_df = df_gps_to_iris(df, "longitude", "latitude")
    assert result_df.CODE_IRIS.values[0] == "132020301"
    assert result_df.CODE_IRIS.values[1] == "130010905"


if __name__ == "__main__":
    test_gps_to_iris()
    test_df_gps_to_iris()
    print("All tests passed.")



"""This module provide functions to enrich data with gps info (e.g. real estates transactions),
with data at the iris level (e.g. poverty rate in an IRIS),
by using IGN geographical dat to join the two datasets.
"""

import pandas as pd
import geopandas as gpd

from iris_insee_utils.gps_coordinates_to_iris import df_gps_to_iris


def merge_gps_dataset_to_iris_dataset(
    df_oi: pd.DataFrame,
    df_enrich: pd.DataFrame,
    iris_year: int = 2018,
    iris_full_info: bool = False,
    df_oi_longlat_colname: tuple[str, str] = (
        "longitude",
        "latitude",
    ),  # TODO: transform to kwargs.
    df_enrich_iriscol_colname: str = None,
) -> gpd.GeoDataFrame:
    """
    Merge the dataset of interest that contains GPS information for each row,
    with another dataset that contains data at the IRIS level.

    Suggested usage: rename the columns of the datasets to 'longitude' and 'latitude' before using this function.

    Parameters
    ----------
    df_oi : pd.DataFrame
        DataFrame containing the dataset of interest with GPS information.
    df_enrich : pd.DataFrame
        DataFrame containing the dataset with IRIS level data to enrich the dataset of interest.
    iris_year : int, optional
        Year of the IRIS data to use. Default is 2018.
    iris_full_info : bool, optional
        If True, return the full information of the IRIS (name, etc.). Default is False.
    df_oi_longlat_colname : tuple[str, str], optional
        Tuple containing the column names for longitude and latitude in the dataset of interest. Default is ('longitude', 'latitude').
    df_enrich_iriscol_colname : str, optional
        Column name in the enrichment dataset that contains the IRIS codes. Default is None.

    Returns
    -------
    geopandas.GeoDataFrame
        A GeoDataFrame with the merged data, containing the original dataset of interest enriched with IRIS level data.

    Examples
    --------
    >>> df_oi = pd.DataFrame({
    ...     'Lieu': ['Mairie de Marseille', 'Site-Mémorial du Camp des Milles'],
    ...     'longitude': [5.369905252590892, 5.382786610618382],
    ...     'latitude': [43.296630332564405, 43.5034655315141]
    ... })
    >>> df_enrich = pd.DataFrame({
    ...     'CODE_IRIS': ['130550101', '130010101'],
    ...     'Some_IRIS_Data': [100, 200]
    ... })
    >>> merge_gps_dataset_to_iris_dataset(df_oi, df_enrich, 2018, False, ('longitude', 'latitude'), 'CODE_IRIS')
    Out[11]: 
                        geometry  index_right INSEE_COM  ...  CODE_IRIS              NOM_IRIS TYP_IRIS
    0  POINT (5.36222 43.41523)        37408     13071  ...  130710101  Cd6-Plan de Campagne        A
    """

    df_oi_iris = df_gps_to_iris(
        df_oi,
        df_oi_longlat_colname[0],
        df_oi_longlat_colname[1],
        iris_year=iris_year,
        iris_full_info=iris_full_info,
    )

    df_enrich = df_enrich.astype({df_enrich_iriscol_colname: "str"})
    print(df_oi_iris)

    return pd.merge(
        df_oi_iris.astype({"CODE_IRIS": "str"}),
        df_enrich,
        left_on="CODE_IRIS",
        right_on=df_enrich_iriscol_colname,
    )



"""This module is for plotting IRIS data."""

import os

from iris_insee_utils.get_iris_contours_data import read_or_download_iris_contour_data
from iris_insee_utils.gps_coordinates_to_iris import gps_to_iris
import pandas as pd
from loguru import logger
import geopandas as gpd
import folium
from tqdm import tqdm
import numpy as np

from loguru import logger


def plot_folium_map(
    iris_year,
    commune_name="Marseille",
    df_oi: pd.DataFrame = None,
    df_enrich: pd.DataFrame = None,
    df_enrich_iriscol_colname=None,
    df_enrich_select_cols=None,
    save_map_path=None,
):
    """
    Plot a folium map of IRIS (Ilots Regroupés pour l'Information Statistique) for a given commune or department number.
                    Parameters
                    ----------
                    iris_year : int
                        The year of the IRIS data to be used.
                    commune_name : str, optional
                        The name of the commune to filter the IRIS data, by default "Marseille".
                    df_oi : pd.DataFrame, optional
                        A DataFrame containing points of interest with latitude and longitude columns, by default None.
                    df_enrich : pd.DataFrame, optional
                        A DataFrame containing additional data to enrich the IRIS data, by default None.
                    df_enrich_iriscol_colname : str, optional
                        The column name in `df_enrich` that corresponds to the IRIS code, by default None.
                    df_enrich_select_cols : list, optional
                        A list of columns in `df_enrich` to be included in the map, by default None.
                    save_map_path : str, optional
                        Absolute path where the map will be saved to an HTML file, by default None.
                    Returns
                    -------
                    folium.Map
                        A Folium map object with the plotted IRIS data and optional points of interest.
                    Raises
                    ------
                    NotImplementedError
                        If `save_map` is True, as this functionality has not been implemented yet.
                    Notes
                    -----
                    - The function reads a parquet file containing IRIS data for the specified year.
                    - The IRIS data is filtered by the specified commune name.
                    - If `df_enrich` is provided, it merges the IRIS data with the enrichment data.
                    - If `df_oi` is provided, it adds markers for points of interest to the map.
                    - The map is centered on the centroid of the filtered IRIS geometries.
                    - The map is returned as a Folium map object.
    """

    df_map = read_or_download_iris_contour_data(iris_year).to_crs(
        epsg=4326
    )  # TODO: add this to the cleaning function
    df_map["NOM_COM"] = (
        df_map.NOM_COM.str.strip()
    )  # remove leading and trailing spaces, TODO: add this to the cleaning function
    df_map = df_map[
        df_map.NOM_COM.str.contains(commune_name, case=True)
    ]  # df_map = df_map.query("NOM_COM == @commune_name")
    m = folium.Map(
        location=[df_map.geometry.centroid.y.mean(), df_map.geometry.centroid.x.mean()],
        zoom_start=12,
    )

    if df_enrich is not None:
        df_enrich = df_enrich.astype({df_enrich_iriscol_colname: "str"})
        df_map = pd.merge(
            df_map.astype({"CODE_IRIS": "str"}),
            df_enrich.loc[:, df_enrich_select_cols + [df_enrich_iriscol_colname]],
            left_on="CODE_IRIS",
            right_on=df_enrich_iriscol_colname,
        )

    for _, row in df_map.iterrows():
        enrich_iris_info = ""
        if df_enrich is not None:
            for column_name, value in row.items():

                if column_name in df_enrich_select_cols:
                    enrich_iris_info += f"{column_name}: {np.round(value,2)}<br>"

        sim_geo = gpd.GeoSeries(row["geometry"])  # .simplify(tolerance=0.001)
        geo_j = sim_geo.to_json()
        geo_j = folium.GeoJson(
            data=geo_j,
            smooth_factor=0.1,
            style_function=lambda x: {"fillColor": "lightblue"},
        )
        iris_base_info = (
            "Code iris: "
            + row["CODE_IRIS"]
            + "<br>"
            + "nom iris: "
            + row["NOM_IRIS"]
            + "<br>"
            + "Commune: "
            + row["NOM_COM"]
            + "<br>"
        )
        folium.Popup(iris_base_info + enrich_iris_info).add_to(geo_j)
        geo_j.add_to(m)

    if df_oi is not None:

        for _, row in df_oi.iterrows():
            location = [row["lat"], row["lon"]]
            tooltip = ""
            for column_name, value in row.items():
                if column_name not in ["lat", "lon"]:
                    tooltip += f"{column_name}: {value}<br>"

            m.add_child(
                folium.Marker(
                    location=[row["lat"], row["lon"]], tooltip=tooltip  # icon=icon,
                )
            )
    if save_map_path is not None:
        script_dir = os.path.dirname(os.path.abspath(__file__))
        logger.info(f"Saving map to {save_map_path}")
        m.save(save_map_path)

    return m


if __name__ == "__main__":
    plot_folium_map(iris_year=2018, commune_name="Nice")



"Will get the contour data for the given year by either reading it from cache in data/transformed or by downloading it from iris_insee_utils_data and recording it in data/transformed."
import geopandas as gpd
import iris_insee_utils
import requests
from loguru import logger
from pathlib import Path

def read_or_download_iris_contour_data(iris_year: int) -> gpd.GeoDataFrame:
    """
    Get the contour data for the given year by either reading it from cache in data/transformed or by downloading it from iris_insee_utils_data and recording it in data/transformed.

    Parameters
    ----------
    iris_year : int
        The year of the IRIS data to be used.

    Returns
    -------
    geopandas.GeoDataFrame
        A GeoDataFrame containing the IRIS contour data for the specified year.
    """
    try:
            df_ign_map = gpd.read_parquet(
                iris_insee_utils.__path__[0] + f"/../data/transformed/iris_{iris_year}.parquet"
            )
    except FileNotFoundError:
        logger.info(f"The file iris_{iris_year}.parquet does not exist. Will try to download it and cache it.")
        url = f"https://github.com/adrienpacifico/iris_insee_utils_data/raw/refs/heads/main/data/primary/iris_{iris_year}.parquet"
        response = requests.get(url)
        response.raise_for_status()

        data_dir = Path(iris_insee_utils.__path__[0]).parent / "data" / "transformed"
        data_dir.mkdir(parents=True, exist_ok=True)
        file_path = data_dir / f"iris_{iris_year}.parquet"

        with open(file_path, "wb") as file:
            file.write(response.content)

        df_ign_map = gpd.read_parquet(file_path)
    return df_ign_map

if __name__ == "__main__":   # pragma: no cover
    for year in range(2018, 2024):
        read_or_download_iris_contour_data(iris_year=year)



import pandas as pd
from iris_insee_utils.merge_geographical_info_to_iris_data import (
    merge_gps_dataset_to_iris_dataset,
)


def test_merge_gps_dataset_to_iris_dataset():
    df_oi = pd.DataFrame(
        {
            "Lieu": ["Mairie de Marseille", "Site-Mémorial du Camp des Milles"],
            "longitude": [5.369905252590892, 5.382786610618382],
            "latitude": [43.296630332564405, 43.5034655315141],
        }
    )

    df_enrich = pd.DataFrame(
        {"CODE_IRIS": ["132020301", "130010905"], "Some_IRIS_Data": [100, 200]}
    )

    result = merge_gps_dataset_to_iris_dataset(
        df_oi, df_enrich, 2018, False, ("longitude", "latitude"), "CODE_IRIS"
    )

    assert isinstance(result, pd.DataFrame)
    assert "Some_IRIS_Data" in result.columns
    assert len(result) == len(df_oi)


if __name__ == "__main__":
    test_merge_gps_dataset_to_iris_dataset()



"""install library pip install -e . if in developer mode"""
from setuptools import setup, find_packages

setup(
    name="iris_insee_utils",
    version="0.0.1",
    url="https://github.com/mypackage.git",
    author="adrienpacifico",
    author_email="adrienpacificopro@gmail.com",
    description="get iris from gps points or address",
    packages=find_packages(),
    # install_requires=['numpy >= 1.11.1', 'matplotlib >= 1.5.1'],
)



from .gps_coordinates_to_iris import gps_to_iris, df_gps_to_iris, gps_to_iris_number
from .iris_map import plot_folium_map
from .merge_geographical_info_to_iris_data import merge_gps_dataset_to_iris_dataset
